@@ -53,17 +53,19 @@
 #'  = list(x = 1:2, y = 3:4)}. It splits the conditioning variables as evenly as
 #'  possible into 2 groups with the x component getting the first 2 variables,
 #'  f1 and f2, and the y component getting the second 2 variables, f3 and f4.
-#'  (if there are an odd number of variables, the x component gets one more
-#'  variable than the y). This means the levels of the x variables, f1 and f2,
-#'  vary across each row; and the levels of the y variables, f3 and f4, vary
-#'  down each column.  Since there are 4 combinations of levels for the x
-#'  variables and 12 for y, this gives a 12 row by 4 column display.
+#'  (if there are an odd number of variables, the x component gets one more variable than
+#'  the y). This means the levels of the x variables, f1 and f2, vary from left to right
+#'  across each row. For the y variables, if \code{as.table = TRUE}, the default, the
+#'  levels of the y variables, f3 and f4, vary from top to bottom down each column;
+#'  otherwise if \code{as.table = FALSE}, from bottom to top up each column. Since there
+#'  are 4 combinations of levels for the x variables and 12 for y, this gives a 12 row by
+#'  4 column display.
 #'
 #'  The panels are displayed in each direction in reverse lexicographic order,
 #'  where the 'alphabets' are the factor levels. This means that the first
 #'  variable changes the fastest; the second the next fastest, and so on. Using
 #'  (i,j) to denote setting in which the first factor is at the ith level and
-#'  the second is at the jth, this translates to:
+#'  the second is at the jth, this translates to (for \code{as.table = TRUE}):
 #'
 #'  \describe{ \item{Row ordering}{ (f1,f2): (1,1), (2,1), (1,2), (2,2)}
 #'  \item{Column ordering from top down}{ (f3, f4): ((1,1), (2,1), (3,1), (1,2),
@@ -85,7 +87,7 @@
 #'
 #'  For the example, this means that the row spacing would look like ('X'
 #'  indicates a panel): XX  XX . And for columns it would be going down: XXX XXX
-#'  XXX  XXX . The spacings can be different for x and y, but this is usually
+#'  XXX XXX . The spacings can be different for x and y, but this is usually
 #'  unnecessary.
 #'
 #'@section Effective xyLayout specification:
@@ -150,6 +152,8 @@
 #'  subset if applicable. If not found in data, or if data is unspecified, the
 #'  variables are looked for in the environment of the formula.
 #'
+#'@param groups The \code{groups} parameter of \code{\link[lattice]{xyplot}}
+#'
 #'@param xyLayout In its most general form, a list with named "x" and "y"
 #'  components, either or both of which can be missing (or NULL). If there are
 #'  \emph{n} conditioning factors and both x and y are given, then
@@ -209,43 +213,51 @@
 #'  previous, since order matters!
 #'
 #'
-#'@note Because xyLayout and the number of levels in the conditioning variables
+#'@note Because 'xyLayout' and the number of levels in the conditioning variables
 #'  determine the plot structure, a 'layout' argument in the call will be
-#'  ignored.
+#'  ignored. Other \code{xyplot} arguments that are ignored are 'skip','between',
+#'  'drop.unused.levels', 'strip','perm.cond',and 'index.cond'. All other \code{xyplot}
+#'  should work as expected.
 #'
 #'@seealso \code{\link[lattice]{lattice}}
 #'
-#'@param spacings A \strong{list} with x and y components that are nondecreasing
-#'  integer sequences. The ith value in the sequence gives the spacing in
-#'  character heights between the sets of panels at each level for the ith
-#'  conditioning variable in the component's direction. See the
-#'  \strong{Overview} for further explanation.
+#'@param spacings A list with x and y components that are nondecreasing
+#'  sequences of positive vaues. The ith value in the sequence gives the spacing
+#'  in character heights between the sets of panels at each level for the ith
+#'  conditioning variable in the component's direction. If either a single
+#'  vector or a list with single component is given, it is replicated. See the
+#'  \strong{Overview} for further explanation. Default is \code{list(x=0:9,
+#'  y=0:9)}
 #'
-#'@param center Logical, default TRUE. If the conditioning factors constitute a
-#'  2-level design with a center point and center is TRUE, a compact trellis
+#'@param center Logical, default FALSE. If the conditioning factors constitute
+#'  a 2-level design with a center point and center is TRUE, a compact trellis
 #'  plot that omits panels for the missing settings of the factors is drawn. If
 #'  FALSE or the design is not of this form, all panels are shown, which may be
 #'  informative in any case to better visualize the design sparsity.
 #'
-#'@param newdata For the \code{lm} method (and objects inheriting from the
-#'  \code{lm} class). The 'newdata' argument for the associated \code{predict}
+#'@param newdata For the \code{lm} method and objects inheriting from the
+#'  \code{lm} class. The 'newdata' argument for the associated \code{predict}
 #'  method. Consult \code{\link[stats]{predict.lm}} for details.
 #'
 #'@param ylab For the \code{lm} method. Optional y axis label for predicted values.
 #'
 #'@param predictArgs For the \code{lm} method. A \strong{list} of optional named
-#'  arguments for the relevant \code{predict} method.
+#'  arguments for the relevant \code{predict} method. These are will be used as part
+#'  of the \code{args} list when invoking the relevant predict method using
+#'  \code{\link{do.call}}.
 #'
 #'@param col Fill color for the data frame method.
 #'
 #'@param \dots Further arguments to panel functions, methods, or
 #'\code{\link[lattice]{xyplot}}.
 #'
-#'@return The formula method returns an object of type c("structured",
-#'  "trellis); or c("doe","structured","trellis") for 2-level designs with a
-#'  center point. This is a trellis object with additional \code{structure} and
-#'  \code{formula} attributes. Other methods may extend the class to allow
-#'  special print/plot methods.
+#'@return The formula method returns an object of type \code{c("structured", "trellis")}; or
+#'  \code{c("doe","structured","trellis")} for 2-level designs with a center point. This is a
+#'  trellis object with additional \code{structure} and \code{formula} attributes. The lm
+#'  method returns a class of \code{c("modelFit","structured","trellis")}. At present, this
+#'  extension is ignored, but future print/plot methods may take advantage of it. The other
+#'  methods return the same object as the formula method.
+#'
 #'
 #'  The \code{structure} attribute provides xyLayout information. It is a list
 #'  with \code{x} and \code{y} components, each of which in turn is a list, one
@@ -265,7 +277,8 @@ strucplot.formula <- function(
   obj
   ## A formula of the kind used in xyplot
   ,data = list()
-  ## as in \code{xyplot}
+  ## as in xyplot
+  ,groups = NULL
   ,xyLayout = list()
   ## A list with named x and/or y integer vector components that when
   ## concatenated give the order in which the conditioning variables are used in
@@ -274,8 +287,10 @@ strucplot.formula <- function(
   ## x (horizontal) and y (vertical) directions, respectively. A missing x
   ## component means only 1 row; a missing y component, only 1 column.
   ,spacings = list(x= 0:9,y= 0:9)
-  ##  list with x and y components that are nondecreasing integer sequences
-##  giving x and y spacings vectors between the panels at the various hierachies
+  ##  list with x and y components that are nondecreasing sequences of positive
+  ##  values giving x and y spacings vectors between the panels at the various
+  ##  levels of the hierachy. If a list with just a single such component or a
+  ##  single vector is given, then it will be replicated.
   ,center=FALSE
  ## if TRUE,only for 2^n factorial designs with a center point. The display will
  ## omit all the empty panels corresponding to the middle levels of the variables
@@ -286,9 +301,10 @@ strucplot.formula <- function(
 )
 {
   ## create data argument containing the variables to evaluate obj
+  groups <- eval(substitute(groups), data, environment(obj))
   spf <- strucParseFormula(obj,data)
   obj <- attr(spf,"form")
-   dots <- list(...)
+  dots <- list(...)
    if(identical(dots[["outer"]],TRUE))stop("'outer = TRUE' option not permitted for strucplot plots")
    d <- spf$condition
    lend <- length(d)
@@ -312,10 +328,11 @@ strucplot.formula <- function(
       obj[[c(3,2)]]<- as.name(nm)
       obj[[2]] <- obj[[c(2,2)]]
       dots[["xlab"]] <- ""
-      dots[["scales"]][["x"]] <- list(draw=FALSE)
+      dots[["scales"]][["x"]] <- list(draw = FALSE)
     }
   }
   xyLayout <- xyLayout(xyLayout, n = lend)
+  spacings <- chkSpacings(spacings)
   levLen <-tryCatch(lapply(xyLayout,
        function(w){
         if(identical(w,integer(0)))1
@@ -323,9 +340,6 @@ strucplot.formula <- function(
       }),
     error=function(e)stop("Can't compute number of levels in conditioning factors"))
 
-  ## Check that spacings are nondecreasing
-  if(!all(sapply(spacings,function(x)all(diff(x)>=0))))
-    stop("spacings must be nondecreasing",call.=FALSE)
   if(!center || !check2lvl(d)){
     center <- FALSE ## to set class attribute of return
     dots[["layout"]] <- sapply(levLen,prod)
@@ -367,14 +381,15 @@ strucplot.formula <- function(
 if(is.null(dots[["as.table"]])) dots[["as.table"]]  <- TRUE
 dots[["strip"]] <- FALSE
 dots[["drop.unused.levels"]] <- FALSE
+dots[["index.cond"]] <- NULL
 ## (To override possible user provided values)
   ## return trellis object for plotting
-structure(do.call(xyplot,c(list(obj,data=data), dots)),
+  ##
+structure(do.call(xyplot,c(list(obj,data=data, groups = groups), dots)),
   class=c(if(center)"doe","structured","trellis","list"),
-  formula = obj,
   structure = lapply(xyLayout,function(i)lapply(d[i],levels)) )
 }
-#
+#obj
 #' @describeIn strucplot Default method prints an error message
 strucplot.default <- function(...)
 stop(paste("No strucplot method",
@@ -385,27 +400,26 @@ stop(paste("No strucplot method",
 strucplot.data.frame <- function(
   obj ## design data frame
   ,col = "darkblue" ## color for filling the panels
+  ,groups = NULL
   ,...
-  ## as in strucplot.formula + arguments for panel.fill
+  ## as in strucplot.formula
   )
 
 {
-  y <- rep(0,nrow(obj))
+  x <- rep(0,nrow(obj))
   dat <- obj
-  obj <- formula(paste0("~y|",paste0(names(obj),collapse="*")))
+  obj <- formula(paste0("~x|",paste0(names(obj),collapse="*")))
   dots <- list(...)
-#   strucplot.formula(obj,data=dat, ...,
-#    scales=list(draw=FALSE),
-#    ylab= "" ,
-#    col=col,
-#    panel=function(y,col,...)if(length(y))lattice::panel.fill(col=col,...) else NA
-#  )
-  dots$panel <- function(y,col,...)if(length(y))lattice::panel.fill(col=col,...)
-              else NA
+  pf <- function(x,col,groups, ...) if(!length(x))NA
+      else panel.fill(col=col)
+  dots$panel <- if(is.null(groups)) pf else {
+          function(x, col, groups,...)
+            panel.superpose(x,groups=groups,col=col, panel.groups = pf,...)
+  }
   dots$scales <- list(draw = FALSE)
   dots$ylab <- ""
   dots$col <- col
-  do.call(strucplot,c(list(obj=obj,data=dat),dots))
+  do.call(strucplot,c(list(obj=obj,data=dat, groups = groups), dots))
 }
 #
 #'@describeIn strucplot Converts to a data frame and calls the data.frame method
